package org.vaadin.sami.autocomplete;

import java.util.List;
import java.util.Map;

import org.vaadin.sami.autocomplete.widgetset.client.ui.VAutoCompleteTextField;

import com.vaadin.event.ShortcutAction.ModifierKey;
import com.vaadin.terminal.PaintException;
import com.vaadin.terminal.PaintTarget;
import com.vaadin.terminal.gwt.server.JsonPaintTarget;
import com.vaadin.ui.ClientWidget;
import com.vaadin.ui.TextField;

@ClientWidget(VAutoCompleteTextField.class)
public class AutoCompleteTextField extends TextField {

	private static final long serialVersionUID = -6051244662590740225L;

	private List<Suggestion> suggestions = null;
	private String suggestWord = "";
	private Suggester suggester = null;
	private int key = ' ';
	private int modifier = ModifierKey.CTRL;
	private int cursorPosition;
	private String text;
	private int textLength;

	public AutoCompleteTextField() {
	}

	public AutoCompleteTextField(String caption) {
		setCaption(caption);
	}

	@Override
	public void paintContent(PaintTarget target) throws PaintException {
		super.paintContent(target);

		target.addVariable(this, "suggestWord", suggestWord == null ? ""
				: suggestWord);
		target.addVariable(this, "cursor", -1);
		target.addVariable(this, "key", getKey());
		target.addVariable(this, "modifier", getModifier());

		if (suggestions != null) {
			String[] captions = new String[suggestions.size()];
			String[] values = new String[suggestions.size()];
			String[] starts = new String[suggestions.size()];
			String[] ends = new String[suggestions.size()];
			int i = 0;
			for (Suggestion s : suggestions) {
				captions[i] = JsonPaintTarget.escapeJSON(s.getCaptionText());
				values[i] = JsonPaintTarget.escapeJSON(s.getValueText());
				starts[i] = "" + s.getStartPosition();
				ends[i++] = "" + s.getEndPosition();
			}
			suggestions = null;

			target.addVariable(this, "captions", captions);
			target.addVariable(this, "values", values);
			target.addVariable(this, "starts", starts);
			target.addVariable(this, "ends", ends);
		}
	}

	@Override
	@SuppressWarnings("unchecked")
	public void changeVariables(Object source, Map variables) {
		super.changeVariables(source, variables);

		suggestions = null;
		if (variables.containsKey("cursor") && suggester != null) {
			cursorPosition = (Integer) variables.get("cursor");
			text = getText();
			textLength = text != null ? text.length() : 0;
			suggestions = suggester.getSuggestions(this, text, cursorPosition);
		}
		if (suggestions != null) {
			requestRepaint();
		}
	}

	public interface Suggester {
		public List<Suggestion> getSuggestions(
				final AutoCompleteTextField source, final String text,
				final int cursorPosition);
	}

	/**
	 * Class for single suggestion for {@link AutoCompleteTextField}.
	 * 
	 * Single suggestion has three attributes: <li>Suggestion value itself. If
	 * user selects this suggestion this value will be inserted to text field.</li>
	 * <li>Display value that is presented inside the suggestion box. <li>Cursor
	 * positions (start and end) for the replacement.
	 */
	public class Suggestion {

		String captionText;
		String valueText;
		int startPosition;
		int endPosition;

		/**
		 * Create new suggestion.
		 * 
		 * @param captionText
		 *            Text to display in the suggestion box
		 * @param valueText
		 *            Value to be inserted into text field if this suggestion is
		 *            selected.
		 * @param valueSuffix
		 *            The part of the value that is located after the current
		 *            cursor position. This is used by the client-side filtering
		 *            method.
		 * @param startPosition
		 *            Start position of the replacement. Must be positive and
		 *            below length of text in textfield. If negative number is
		 *            given the current cursor position is used.
		 * @param endPosition
		 *            End position of the replacement. Must be positive and
		 *            below length of text in textfield. If negative number is
		 *            given the current cursor position is used.
		 */
		public Suggestion(String captionText, String valueText,
				String valueSuffix, int startPosition, int endPosition) {
			super();
			this.captionText = captionText;
			this.valueText = valueText;
			this.startPosition = startPosition < 0 ? cursorPosition
					: startPosition;
			this.endPosition = endPosition < 0 ? cursorPosition : endPosition;

			// Text length check
			int l = getValue().toString().length();
			if (startPosition > l) {
				startPosition = l;
			}
			if (endPosition > l) {
				endPosition = l;
			}
		}

		public String getCaptionText() {
			return captionText;
		}

		public String getValueText() {
			return valueText;
		}

		public int getStartPosition() {
			return startPosition;
		}

		public int getEndPosition() {
			return endPosition;
		}

	}

	public Suggester getSuggester() {
		return suggester;
	}

	public void setSuggester(Suggester suggester) {
		this.suggester = suggester;
	}

	public void setKey(int key) {
		this.key = key;
	}

	public int getKey() {
		return key;
	}

	public void setModifier(int modifier) {
		this.modifier = modifier;
	}

	public int getModifier() {
		return modifier;
	}

	public String getText() {
		return (String) getValue();
	}

	/**
	 * Factory function for creating new Suggestion instances.
	 * 
	 * @param valueText
	 * @param displayText
	 * @param startPosition
	 * @param endPosition
	 * @return new {@link Suggestion} instance
	 */
	public Suggestion createSuggestion(String displayText, String valueText,
			String valueSuffix, int startPosition, int endPosition) {
		return new Suggestion(displayText, valueText, valueSuffix,
				startPosition, endPosition);
	}

	public int getCursorPosition() {
		return cursorPosition;
	}

	public int getTextLength() {
		return textLength;
	}

}
